@@ -17,6 +17,9 @@
 import torch
 from packaging.version import Version
 import os
+import math
+import functools
+from typing import Optional
 torch_nn_functional_cross_entropy = torch.nn.functional.cross_entropy
 from triton import __version__ as triton_version
 from . import DEVICE_TYPE
@@ -63,10 +66,11 @@ __all__ = [
     "fused_linear_cross_entropy",
     "fast_linear_cross_entropy",
     "_unsloth_get_batch_samples",
-    "unsloth_compiled_fused_ce_loss_function",
-    "unsloth_compiled_ce_loss_function",
+    "unsloth_fused_ce_loss",
 ]
 
+from .fused_losses import unsloth_fused_ce_loss
+
 def patch_loss_functions(_fast_cross_entropy_loss, torch_compile = True):
     # All Unsloth Zoo code licensed under LGPLv3
     try:
@@ -349,143 +353,6 @@ def _unsloth_get_batch_samples(self, epoch_iterator, num_batches, device = None,
     return batch_samples, num_items_in_batch
 pass
 
-
-def unsloth_compiled_ce_loss_function(
-    output_logits : torch.Tensor,
-    output_labels : torch.Tensor,
-    logit_scale_multiply : float = 0,
-    logit_scale_divide : float = 0,
-    logit_softcapping : float = 0,
-    vocab_size : int = 0,
-    n_items : int = 0,
-    mask : torch.Tensor = None,
-    requires_grad_ : bool = False,
-):
-    device = output_logits.device
-    if logit_scale_multiply != 0:
-        output_logits = output_logits * logit_scale_multiply
-    if logit_scale_divide != 0:
-        output_logits = output_logits / logit_scale_divide
-    if logit_softcapping != 0:
-        output_logits = output_logits / logit_softcapping
-        output_logits = torch.tanh(output_logits)
-        output_logits = output_logits * logit_softcapping
-
-    shift_logits = output_logits
-    shift_labels = torch.empty_like(output_labels, device = device)
-    shift_labels[..., :-1] = output_labels[..., 1:]
-    if mask is not None:
-        mask = mask.to(device = device)
-        shift_labels[..., :-1][mask[..., 1:] == 0] = -100
-    pass
-    shift_labels[..., -1] = -100
-    # shift_logits = output_logits[..., :-1, :].float().contiguous()
-    # shift_labels = output_labels[..., 1:].contiguous()
-
-    shift_logits = shift_logits.view(-1, vocab_size)
-    shift_labels = shift_labels.view(-1)
-
-    n_chunks = int(torch.ceil((torch.tensor(vocab_size) / 262144) * 8))
-    if requires_grad_: n_chunks += 2
-    __shift_logits = torch.chunk(shift_logits, n_chunks, dim = 0)
-    __shift_labels = torch.chunk(shift_labels, n_chunks, dim = 0)
-    loss = 0.0
-    for (_shift_logits, _shift_labels) in zip(__shift_logits, __shift_labels):
-        loss += torch.nn.functional.cross_entropy(
-            input  = _shift_logits.float().contiguous(),
-            target = _shift_labels.contiguous(),
-            reduction = 'sum',
-        )
-    pass
-    if n_items != 0:
-        loss = loss / n_items
-    else:
-        loss = loss / (shift_labels != -100).sum()
-    return loss
-pass
-unsloth_compiled_ce_loss_function = torch.compile(
-    unsloth_compiled_ce_loss_function,
-    fullgraph = False,
-    dynamic = True,
-    options = torch_compile_options,
-)
-
-
-def unsloth_compiled_fused_ce_loss_function(
-    hidden_states : torch.Tensor,
-    lm_head_weight : torch.Tensor,
-    lm_head_bias : torch.Tensor,
-    output_labels : torch.Tensor,
-    logit_scale_multiply : float = 0,
-    logit_scale_divide : float = 0,
-    logit_softcapping : float = 0,
-    vocab_size : int = 0,
-    n_items : int = 0,
-    mask : torch.Tensor = None,
-    requires_grad_ : bool = False,
-):
-    device = lm_head_weight.device
-
-    # Get shifted labels first
-    shift_labels = torch.empty_like(output_labels, device = device)
-    shift_labels[..., :-1] = output_labels[..., 1:]
-    if mask is not None:
-        mask = mask.to(device = device)
-        shift_labels[..., :-1][mask[..., 1:] == 0] = -100
-    pass
-    shift_labels[..., -1] = -100
-    shift_labels = shift_labels.view(-1)
-
-    # Decide on chunk size
-    n_chunks = int(torch.ceil((torch.tensor(vocab_size) / 262144) * 8))
-    if requires_grad_: n_chunks += 2
-
-    # Chunk hidden_states and labels
-    bsz, qlen, hd = hidden_states.shape
-    hidden_states = hidden_states.view(-1, hd)
-    __shift_states = torch.chunk(hidden_states, n_chunks, dim = 0)
-    __shift_labels = torch.chunk(shift_labels,  n_chunks, dim = 0)
-    loss = 0.0
-
-    # Chunk hidden states and logits
-    for (_shift_states, _shift_labels) in zip(__shift_states, __shift_labels):
-
-        _shift_logits = torch.nn.functional.linear(
-            _shift_states.to(lm_head_weight.device),
-            lm_head_weight,
-            lm_head_bias,
-        )
-
-        # Apply softcapping and other functions
-        if logit_scale_multiply != 0:
-            _shift_logits = _shift_logits * logit_scale_multiply
-        if logit_scale_divide != 0:
-            _shift_logits = _shift_logits / logit_scale_divide
-        if logit_softcapping != 0:
-            _shift_logits = _shift_logits / logit_softcapping
-            _shift_logits = torch.tanh(_shift_logits)
-            _shift_logits = _shift_logits * logit_softcapping
-
-        loss += torch.nn.functional.cross_entropy(
-            input  = _shift_logits.float().contiguous(),
-            target = _shift_labels.contiguous(),
-            reduction = 'sum',
-        )
-    pass
-
-    if n_items != 0:
-        loss = loss / n_items
-    else:
-        loss = loss / (shift_labels != -100).sum()
-    return loss
-pass
-unsloth_compiled_fused_ce_loss_function = torch.compile(
-    unsloth_compiled_fused_ce_loss_function,
-    fullgraph = False,
-    dynamic = True,
-    options = torch_compile_options,
-)
-
 # Unsloth Zoo - Utilities for Unsloth
 # Copyright 2023-present Daniel Han-Chen, Michael Han-Chen & the Unsloth team. All rights reserved.
 #
