@@ -18,7 +18,7 @@ from typing import Any, List, Optional, Tuple, Union, Dict, Set, Callable
 import torch
 import torch.nn as nn
 import os
-from .common import TEMPORARY_PATCHES, torch_compile_options
+from .common import TEMPORARY_PATCHES, torch_compile
 from .utils import (
     patch_function,
     process_output_options,
@@ -302,7 +302,7 @@ def patch_Gemma3MLP():
         gate_proj_out = self.gate_proj(x)
         up_proj_out = self.up_proj(x)
 
-        #  Upcast to fp32
+        # Upcast to fp32
         gate_proj_fp32 = gate_proj_out.to(torch.float32)
         up_proj_fp32 = up_proj_out.to(torch.float32)
         activated_fp32 = self.act_fn(gate_proj_fp32) # Activation in fp32
@@ -376,7 +376,7 @@ def patch_Gemma3Attention():
             attn_mask_for_sdpa,
         )
     pass
-    prepare = torch.compile(prepare, fullgraph = True, dynamic = True, options = torch_compile_options)
+    prepare = torch_compile(prepare, fullgraph = True, dynamic = True)
 
     def forward(
         self,
@@ -562,7 +562,7 @@ def patch_Gemma3Attention_normal():
             attn_mask_for_sdpa,
         )
     pass
-    prepare = torch.compile(prepare, fullgraph = True, dynamic = True, options = torch_compile_options)
+    prepare = torch_compile(prepare, fullgraph = True, dynamic = True)
 
     def forward(
         self,
