@@ -95,7 +95,7 @@ def prepare_model_for_training(
     train_layernorms           : Optional[bool] = False,
     train_embedding            : Optional[bool] = False,
     train_lm_head              : Optional[bool] = False,
-    float32_mixed_precision    : Optional[bool] = False,
+    float32_mixed_precision    : Optional[bool] = True,
 ) -> Any:
     # All Unsloth Zoo code licensed under LGPLv3
     assert(use_gradient_checkpointing in (True, False, "unsloth",))
@@ -154,12 +154,8 @@ def prepare_model_for_training(
         # Upcast to float32 if needed
         if requires_grad:
             name = name.replace("base_model", "model", 1)
-            layer_number = re.search(r"\.[\d]{1,}\.", name)
-            if layer_number is not None:
-                # Convert .0. to [0]
-                layer_number = layer_number.group(0)
-                name = name.replace(layer_number, f"[{layer_number[1:-1]}].")
-            pass
+            while re.search(r'\.(\d+)\.', name) is not None:
+                name = re.sub(r'\.(\d+)\.', r'[\1].', name)
             name = name.replace(".weight", "", 1)
             dtype = torch.float32 if upcast else mixed_precision_dtype
             try:
@@ -169,6 +165,18 @@ def prepare_model_for_training(
                 # Maybe model.model
                 exec(f"model.{name}.to({str(dtype)})")
         pass
+
+        if ('norm.' in name or '_layernorm' in name) and os.environ.get("UNSLOTH_UPCAST_LAYERNORM", "0") == "1":
+            try:
+                name = name.replace("base_model", "model", 1)
+                while re.search(r'\.(\d+)\.', name) is not None:
+                    name = re.sub(r'\.(\d+)\.', r'[\1].', name)
+                name = name.replace(".weight", "", 1)
+                # Try original name
+                exec(f"{name}.to({str(torch.float32)})")
+            except:
+                # Maybe model.model
+                exec(f"model.{name}.to({str(torch.float32)})")
     pass
 
     # Gradient checkpointing
